%% Calibration code %%
%This file computes moments with lefskewed distributions, generates sd's
%and correlations among velocity and the other variables only
%This single file computes the cases for sigma = 1 2, but for RIGHT
%SKEWED DISTRIBUTION

% date: 04-08-2020
% author: lucas rosso

clear all; close all; clc;
addpath C:\dynare\5.0\matlab;

path = 'D:\Dropbox\Brennan\new projects\money demand new\revision\data\main data.xls';
data = xlsread(path, 12,'B3:I254');

datas = data;            %--data for FULL SAMPLE

NN = size(datas,1);


%%%% MOMENTS FROM THE DATA USED FOR ESTIMATION(SEE MAIN DATA.XLS) %%%
gy_mean = mean(datas(:,1));
gm_mean = mean(datas(:,2));
sd_gy = std(datas(:,1));
sd_gm = std(datas(:,2));

corr_gy_gyl = corr(datas(2:NN,1),datas(1:NN-1,1));
corr_gm_gml = corr(datas(2:NN,2),datas(1:NN-1,2));
corr_gy_gml = corr(datas(2:NN,1),datas(1:NN-1,2));
corr_gm_gyl = corr(datas(2:NN,2),datas(1:NN-1,1));
corr_gy_gm  = corr(datas(:,1),datas(:,2));

%%%% MOMENTS FROM THE DATA USED FOR COMPUTATION %%%
gy       = datas(:,1);
gm       = datas(:,2);
velocity = datas(:,8);
rb       = datas(:,5);
py       = datas(:,3);
re       = datas(:,6);
qe       = datas(:,7);
i        = datas(:,4);

ret_mean = 0.01745;

py_bar = (1+gm_mean)/(1+gy_mean)-1;
Rm_bar = 1/(1+py_bar);
qe_bar = 1/((ret_mean-gy_mean)*(1+py_bar));
Qb_bar = 1/((1+py_bar)*(1+ret_mean));
i_bar  = 1/Qb_bar-1;

%removing trends

%py = py-hpfilter(py,1600)+py_bar;
py = detrend(py,0)+py_bar;

%i = i-hpfilter(i,1600)+i_bar;                 %---HP filtering i..maybe strange...
i = detrend(i,0)+i_bar;

%rb = rb-hpfilter(rb,1600)+ret_mean;
rb = detrend(rb,0)+ret_mean;


%velocity = detrend(velocity,3);
velocity= velocity-hpfilter(velocity,1600);



sd_i        = std(i);
sd_py       = std(py);
sd_velocity = std(velocity);
sd_rb       = std(rb);
sd_py       = std(py);


%---constructing re as in the model
qe = qe-hpfilter(qe,1600);
qe_hat = qe+qe_bar;

re = (qe_hat(2:size(qe_hat)).*(1+datas(2:size(qe_hat),1))+1./(1+datas(2:size(qe_hat),3)))./qe_hat(1:size(qe_hat)-1)-1;
sd_re = std(re);



%--Here I compute relevant statistics in tables
filename = 'D:\Tablasdistributions.xlsx';           %--excel for full sample

table1 = [corr_gy_gyl;corr_gy_gml;sd_gy;corr_gm_gml;corr_gm_gyl;sd_gm;corr_gy_gm];
table4 = [sd_velocity;sd_i;sd_rb;sd_re;sd_py]*100;
%writematrix(table1,filename,'Sheet',3,'Range','B2');
writematrix(table4,filename,'Sheet',4,'Range','B2');


global data_moments oo_
% parameter guess full sample

rho_y   = -0.078581006392428;
sigma_y = 0.009631131453605;
rho_m   = 0.576210599307085;

sigma_m = 0.008467988268384;
rho_my  = 0.094801246862030;

rho_ym  = 0.151039136048165;
sigma_ym= -0.653600078611112; 
f = 0.155010710792391;


vector_params = [rho_y,sigma_y,rho_m,sigma_m,rho_my,rho_ym,sigma_ym,f];

data_moments = [sd_gy,corr_gy_gyl,sd_gm,corr_gm_gml,corr_gm_gyl,corr_gy_gml,corr_gy_gm]; 



LB = [-0.95  ,1e-10  ,-0.9  ,1e-10  ,-2    ,-0.2  ,-0.99     ,0.01   ];
UB = [0.95  ,0.025   ,0.9  ,0.03   ,5     ,0.5   ,0.99      ,0.25    ];
%UB = [0.95  ,0.01   ,0.9  ,0.01   ,5     ,0.2   ,0.99      ,0.2    ];

options = optimset('PlotFcns',@optimplotfval,'Display','iter','MaxFunEvals',10000,'MaxIter',5000);

%This vector a is coming from ss in gauss

%{
%--a for full sample sigma =1

a = [0.98666397       0.50284216       0.70911364        1.0000000        1.4102112        1.9886956       0.33949492       0.28509865       0.20838842       0.10021080      0.066807201        1.0000000      0.013860104     0.0039062522      0.022709132        76.564500       0.97319989        1.0174500        1.0174500       0.99018222        16.126571        22.668573        31.822835        44.592391       0.63880735       0.49228376       0.28725242     0.0012480568      0.022410823        64.145453     0.0099151204        73.109913        4114.6392 ];
%}


%--a for full sample sigma =2

a = [ 0.99049944       0.36183368       0.60152613        1.0000000        1.6624382        2.7637007       0.32917464       0.28697818       0.21682918       0.10021080      0.066807201        2.0000000      0.013860104     0.0039062522      0.021478124        76.368887       0.97319989        1.0174500        1.0174500       0.99018222        20.492312        26.353557        33.866195        43.479283       0.56589200       0.44172772       0.28258041      0.078937298      0.021195985        4425.3942     0.0099151204        73.109913        19584113. ];



optimal_params = fminsearchbnd(@(vector_params)Loss(vector_params,a),vector_params, LB, UB, options)

%Re(1) Rb(2) py(16) w qe gy(19) gm(20) v(21) i(22) r(23); 

Table = [sqrt(oo_.var(19,19)), data_moments(1);
        oo_.autocorr{1}(19,19), data_moments(2);
        sqrt(oo_.var(20,20)), data_moments(3);
        oo_.autocorr{1}(20,20), data_moments(4);
        oo_.autocorr{1}(20,19), data_moments(5);
        oo_.autocorr{1}(19,20), data_moments(6);
        oo_.contemporaneous_correlation(19,20), data_moments(7)];

save Results optimal_params Table

table2 = [optimal_params].';
%writematrix(table2,filename,'Sheet',2,'Range','B2');

%--First values in steady state
table1 = [a(1);a(33);a(2:6)';a(29);zb_bar;w_bar;a(30)];
%writematrix(table1,filename,'Sheet',1,'Range','B2');

%Re(1) Rb(2) py(16) w qe gy(19) gm(20) v(21) i(22) r(23); 

%--variances
MV = oo_.var;
CC = oo_.contemporaneous_correlation;
AC = oo_.autocorr{1,1};


sdv  = sqrt(MV(21,21));
sdi  = sqrt(MV(22,22));
sdr  = sqrt(MV(23,23));
sdRe = sqrt(MV(1,1));
sdpy = sqrt(MV(16,16));

%Re(1) Rb(2) py(16) w qe gy(19) gm(20) v(21) i(22) r(23); 

%---"estimated" statistics---%
cgygyl = AC(19,19);
cgygml = AC(19,20);
sdgy   = sqrt(MV(19,19));
cgmgml = AC(20,20);
cgmgyl = AC(20,19);
sdgm   = sqrt(MV(20,20));
cgygm  = CC(19,20);

table3 = [cgygyl;cgygml;sdgy;cgmgml;cgmgyl;sdgm;cgygm];
%writematrix(table3,filename,'Sheet',3,'Range','C2');

%---standard deviations---%
%Re(1) py(8) gy(11) gm(12) i(13) r(14);

table4 = [sdv;sdi;sdr;sdRe;sdpy]*100;

%EXPORTING FOR SIGMA 1 2
writematrix(table4,filename,'Sheet',4,'Range','I2');



%--correlations
%Re(1) Rb(2) py(16) w qe gy(19) gm(20) v(21) i(22) r(23); 

AC1 = oo_.autocorr{1,1};
AC2 = oo_.autocorr{1,2};
AC3 = oo_.autocorr{1,3};
AC4 = oo_.autocorr{1,4};

corr_velocity_velocity_model =[AC4(21,21); AC3(21,21); AC2(21,21); AC1(21,21); CC(21,21); AC1(21,21); AC2(21,21); AC3(21,21); AC4(21,21)]; 
[corr_velocity_velocity_data,lags] = crosscorr(velocity,velocity,4);

corr_velocity_gy_model =[AC4(21,19); AC3(21,19); AC2(21,19); AC1(21,19); CC(21,19); AC1(19,21); AC2(19,21); AC3(19,21); AC4(19,21)]; 
[corr_velocity_gy_data,lags] = crosscorr(velocity,gy,4);

corr_velocity_gm_model =[AC4(21,20); AC3(21,20); AC2(21,20); AC1(21,20); CC(21,20); AC1(20,21); AC2(20,21); AC3(20,21); AC4(20,21)]; 
[corr_velocity_gm_data,lags] = crosscorr(velocity,gm,4);

corr_velocity_i_model =[AC4(21,22); AC3(21,22); AC2(21,22); AC1(21,22); CC(21,22); AC1(22,21); AC2(22,21); AC3(22,21); AC4(22,21)]; 
[corr_velocity_i_data,lags] = crosscorr(velocity,i,4);

corr_velocity_rb_model =[AC4(21,23); AC3(21,23); AC2(21,23); AC1(21,23); CC(21,23); AC1(23,21); AC2(23,21); AC3(23,21); AC4(23,21)]; 
[corr_velocity_rb_data,lags] = crosscorr(velocity,rb,4);

corr_velocity_re_model =[AC4(21,1); AC3(21,1); AC2(21,1); AC1(21,1); CC(21,1); AC1(1,21); AC2(1,21); AC3(1,21); AC4(1,21)]; 
[corr_velocity_re_data,lags] = crosscorr(velocity(2:NN),re,4);

corr_velocity_py_model =[AC4(21,16); AC3(21,16); AC2(21,16); AC1(21,16); CC(21,16); AC1(16,21); AC2(16,21); AC3(16,21); AC4(16,21)]; 
[corr_velocity_py_data,lags] = crosscorr(velocity,py,4);


figure
stem(lags,[corr_velocity_velocity_data],'r','filled');
hold on
stem(lags,[corr_velocity_velocity_model],'-b');
title('$corr(\vartheta_t,\vartheta_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_velo_b.pdf'])
exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_velo_c.pdf'])

figure
stem(lags,[corr_velocity_gy_data],'r','filled');
hold on
stem(lags,[corr_velocity_gy_model],'-b');
title('$corr(\vartheta_t,\gamma_{yt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_gy_b.pdf'])
exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_gy_c.pdf'])


figure
stem(lags,[corr_velocity_gm_data],'r','filled');
hold on
stem(lags,[corr_velocity_gm_model],'-b');
title('$corr(\vartheta_t,\gamma_{mt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_gm_b.pdf'])
exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_gm_c.pdf'])


figure
stem(lags,[corr_velocity_i_data],'r','filled');
hold on
stem(lags,[corr_velocity_i_model],'-b');
title('$corr(\vartheta_t,i_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_i_b.pdf'])
exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_i_c.pdf'])


figure
stem(lags,[corr_velocity_rb_data],'r','filled');
hold on
stem(lags,[corr_velocity_rb_model],'-b');
title('$corr(\vartheta_t,r_{bt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_rb_b.pdf'])
exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_rb_c.pdf'])


figure
stem(lags,[corr_velocity_re_data],'r','filled');
hold on
stem(lags,[corr_velocity_re_model],'-b');
title('$corr(\vartheta_t,r_{et\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_re_b.pdf'])
exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_re_c.pdf'])


figure
stem(lags,[corr_velocity_py_data],'r','filled');
hold on
stem(lags,[corr_velocity_py_model],'-b');
title('$corr(\vartheta_t,\pi_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_py_b.pdf'])
exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\r_py_c.pdf'])


%Re(1) Rb(2) py(16) w qe gy(19) gm(20) v(21) i(22) r(23); 


