%% Calibration code %%

% date: 04-08-2020
% author: lucas rosso

clear all; close all; clc;

path = 'D:\Dropbox\Brennan\new projects\money demand new\revision\data\main data.xls';
data = xlsread(path, 12,'B3:I254');
datas = data;            %--data for FULL SAMPLE

NN = size(datas,1);


%%%% MOMENTS FROM THE DATA USED FOR ESTIMATION(SEE MAIN DATA.XLS) %%%
gy_mean = mean(datas(:,1));
gm_mean = mean(datas(:,2));
sd_gy = std(datas(:,1));
sd_gm = std(datas(:,2));

corr_gy_gyl = corr(datas(2:NN,1),datas(1:NN-1,1));
corr_gm_gml = corr(datas(2:NN,2),datas(1:NN-1,2));
corr_gy_gml = corr(datas(2:NN,1),datas(1:NN-1,2));
corr_gm_gyl = corr(datas(2:NN,2),datas(1:NN-1,1));
corr_gy_gm  = corr(datas(:,1),datas(:,2));

%%%% MOMENTS FROM THE DATA USED FOR COMPUTATION %%%
gy       = datas(:,1);
gm       = datas(:,2);
velocity = datas(:,8);
rb       = datas(:,5);
py       = datas(:,3);
re       = datas(:,6);
qe       = datas(:,7);
i        = datas(:,4);

ret_mean = 0.01745;

py_bar = (1+gm_mean)/(1+gy_mean)-1;
Rm_bar = 1/(1+py_bar);
qe_bar = 1/((ret_mean-gy_mean)*(1+py_bar));
Qb_bar = 1/((1+py_bar)*(1+ret_mean));
i_bar  = 1/Qb_bar-1;

%removing trends

py = detrend(py,0)+py_bar;

i = detrend(i,0)+i_bar;

rb = detrend(rb,0)+ret_mean;

velocity= velocity-hpfilter(velocity,1600);

sd_i        = std(i);
sd_py       = std(py);
sd_velocity = std(velocity);
sd_rb       = std(rb);
sd_py       = std(py);


%---constructing re as in the model
qe = qe-hpfilter(qe,1600);
qe_hat = qe+qe_bar;

re = (qe_hat(2:size(qe_hat)).*(1+datas(2:size(qe_hat),1))+1./(1+datas(2:size(qe_hat),3)))./qe_hat(1:size(qe_hat)-1)-1;
sd_re = std(re);


%--Here I compute relevant statistics in tables
filename = 'D:\Tablasfull.xlsx';           %--excel for full sample

table1 = [corr_gy_gyl;corr_gy_gml;sd_gy;corr_gm_gml;corr_gm_gyl;sd_gm;corr_gy_gm];
table4 = [sd_velocity;sd_i;sd_rb;sd_re;sd_py]*100;
%writematrix(table1,filename,'Sheet',3,'Range','F2');
%writematrix(table4,filename,'Sheet',4,'Range','F2');


global data_moments oo_
% parameter guess full sample

rho_y   = -0.078581473051902;
sigma_y = 0.009631028790270;
rho_m   = 0.448859346575175;

sigma_m = 0.009182800623003;
rho_my  = 0.080406349935926;

rho_ym  = 0.151038118368370;
sigma_ym= -0.602716909210906; 
f = 0.036592537558139;



vector_params = [rho_y,sigma_y,rho_m,sigma_m,rho_my,rho_ym,sigma_ym,f];

data_moments = [sd_gy,corr_gy_gyl,sd_gm,corr_gm_gml,corr_gm_gyl,corr_gy_gml,corr_gy_gm]; 

LB = [-0.95  ,1e-10  ,-0.9  ,1e-10  ,-2    ,-0.2  ,-0.9     ,0.01   ];
UB = [0.95  ,0.01   ,0.9  ,0.01   ,5     ,0.2   ,0.9      ,0.4    ];


%LB = [-0.95  ,1e-10  ,-0.9  ,1e-10  ,-2    ,-0.2  ,-0.9     ,0.01   ];
%UB = [0.95  ,0.025   ,0.9  ,0.03   ,5     ,0.5   ,0.99      ,0.2    ];
%UB = [0.95  ,0.01   ,0.9  ,0.01   ,5     ,0.2   ,0.9      ,0.4    ];

options = optimset('PlotFcns',@optimplotfval,'Display','iter','MaxFunEvals',10000,'MaxIter',5000);

%This vector a is coming from ss in gauss

%--a for full sample sigma =2

a = [0.99051739       0.36183368       0.60152613        1.0000000        1.6624382        2.7637007      0.066807201       0.24173034       0.38292492       0.24173034      0.066807201        2.0000000      0.013860104     0.0039062522      0.018339963        75.874713       0.97319989        1.0174500        1.0174500       0.99018222        23.976676        30.846241        39.658931        50.948083       0.56625550       0.44198323       0.28255931      0.078335526      0.018099047        6096.6657     0.0099151204        73.109913       0.25835395 ];

optimal_params = fminsearchbnd(@(vector_params)Loss(vector_params,a),vector_params, LB, UB, options)


%Re(1) Rb(2) py(16) w qe gy(19) gm(20) v(21) i(22) r(23); 

Table = [sqrt(oo_.var(19,19)), data_moments(1);
        oo_.autocorr{1}(19,19), data_moments(2);
        sqrt(oo_.var(20,20)), data_moments(3);
        oo_.autocorr{1}(20,20), data_moments(4);
        oo_.autocorr{1}(20,19), data_moments(5);
        oo_.autocorr{1}(19,20), data_moments(6);
        oo_.contemporaneous_correlation(19,20), data_moments(7)];


save Results optimal_params Table

table2 = [optimal_params].';
%writematrix(table2,filename,'Sheet',2,'Range','E2');

%--First values in steady state
table1 = [a(1);a(33);a(2:6)';a(29);zb_bar;w_bar;a(30)];
%writematrix(table1,filename,'Sheet',1,'Range','E2');


%--variances
MV = oo_.var;
CC = oo_.contemporaneous_correlation;
AC = oo_.autocorr{1,1};


sdv  = sqrt(MV(21,21));
sdi  = sqrt(MV(22,22));
sdr  = sqrt(MV(23,23));
sdRe = sqrt(MV(1,1));
sdpy = sqrt(MV(16,16));

%Re(1) Rb(2) py(16) w qe gy(19) gm(20) v(21) i(22) r(23); 

%---"estimated" statistics---%
cgygyl = AC(19,19);
cgygml = AC(19,20);
sdgy   = sqrt(MV(19,19));
cgmgml = AC(20,20);
cgmgyl = AC(20,19);
sdgm   = sqrt(MV(20,20));
cgygm  = CC(19,20);

table3 = [cgygyl;cgygml;sdgy;cgmgml;cgmgyl;sdgm;cgygm];
%writematrix(table3,filename,'Sheet',3,'Range','F2');

%---standard deviations---%
%Re(1) py(8) gy(11) gm(12) i(13) r(14);

table4 = [sdv;sdi;sdr;sdRe;sdpy]*100;
%writematrix(table4,filename,'Sheet',4,'Range','F2');




%--correlations
%Re(1) Rb(2) py(16) w qe gy(19) gm(20) v(21) i(22) r(23); 

AC1 = oo_.autocorr{1,1};
AC2 = oo_.autocorr{1,2};
AC3 = oo_.autocorr{1,3};
AC4 = oo_.autocorr{1,4};

corr_velocity_velocity_model =[AC4(21,21); AC3(21,21); AC2(21,21); AC1(21,21); CC(21,21); AC1(21,21); AC2(21,21); AC3(21,21); AC4(21,21)]; 
[corr_velocity_velocity_data,lags] = crosscorr(velocity,velocity,4);

corr_velocity_gy_model =[AC4(21,19); AC3(21,19); AC2(21,19); AC1(21,19); CC(21,19); AC1(19,21); AC2(19,21); AC3(19,21); AC4(19,21)]; 
[corr_velocity_gy_data,lags] = crosscorr(velocity,gy,4);

corr_velocity_gm_model =[AC4(21,20); AC3(21,20); AC2(21,20); AC1(21,20); CC(21,20); AC1(20,21); AC2(20,21); AC3(20,21); AC4(20,21)]; 
[corr_velocity_gm_data,lags] = crosscorr(velocity,gm,4);

corr_velocity_i_model =[AC4(21,22); AC3(21,22); AC2(21,22); AC1(21,22); CC(21,22); AC1(22,21); AC2(22,21); AC3(22,21); AC4(22,21)]; 
[corr_velocity_i_data,lags] = crosscorr(velocity,i,4);

corr_velocity_rb_model =[AC4(21,23); AC3(21,23); AC2(21,23); AC1(21,23); CC(21,23); AC1(23,21); AC2(23,21); AC3(23,21); AC4(23,21)]; 
[corr_velocity_rb_data,lags] = crosscorr(velocity,rb,4);

corr_velocity_re_model =[AC4(21,1); AC3(21,1); AC2(21,1); AC1(21,1); CC(21,1); AC1(1,21); AC2(1,21); AC3(1,21); AC4(1,21)]; 
[corr_velocity_re_data,lags] = crosscorr(velocity(2:NN),re,4);

corr_velocity_py_model =[AC4(21,16); AC3(21,16); AC2(21,16); AC1(21,16); CC(21,16); AC1(16,21); AC2(16,21); AC3(16,21); AC4(16,21)]; 
[corr_velocity_py_data,lags] = crosscorr(velocity,py,4);

figure
stem(lags,[corr_velocity_velocity_data],'r','filled');
hold on
stem(lags,[corr_velocity_velocity_model],'-b');
title('$corr(\vartheta_t,\vartheta_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\velo_c.pdf'])

figure
stem(lags,[corr_velocity_gy_data],'r','filled');
hold on
stem(lags,[corr_velocity_gy_model],'-b');
title('$corr(\vartheta_t,\gamma_{yt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\gy_c.pdf'])

figure
stem(lags,[corr_velocity_gm_data],'r','filled');
hold on
stem(lags,[corr_velocity_gm_model],'-b');
title('$corr(\vartheta_t,\gamma_{mt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\gm_c.pdf'])

figure
stem(lags,[corr_velocity_i_data],'r','filled');
hold on
stem(lags,[corr_velocity_i_model],'-b');
title('$corr(\vartheta_t,i_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\i_c.pdf'])

figure
stem(lags,[corr_velocity_rb_data],'r','filled');
hold on
stem(lags,[corr_velocity_rb_model],'-b');
title('$corr(\vartheta_t,r_{bt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\rb_c.pdf'])

figure
stem(lags,[corr_velocity_re_data],'r','filled');
hold on
stem(lags,[corr_velocity_re_model],'-b');
title('$corr(\vartheta_t,r_{et\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\re_c.pdf'])

figure
stem(lags,[corr_velocity_py_data],'r','filled');
hold on
stem(lags,[corr_velocity_py_model],'-b');
title('$corr(\vartheta_t,\pi_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);
%exportgraphics(gcf,['D:\Dropbox\Brennan\new projects\money demand new\revision\text\py_c.pdf'])

%Re(1) Rb(2) py(16) w qe gy(19) gm(20) v(21) i(22) r(23); 


corr_gy_gy_model =[AC4(19,19); AC3(19,19); AC2(19,19); AC1(19,19); CC(19,19); AC1(19,19); AC2(19,19); AC3(19,19); AC4(19,19)]; 
[corr_gy_gy_data,lags] = crosscorr(gy,gy,4);

corr_gy_gm_model =[AC4(19,20); AC3(19,20); AC2(19,20); AC1(19,20); CC(19,20); AC1(20,19); AC2(20,19); AC3(20,19); AC4(20,19)]; 
[corr_gy_gm_data,lags] = crosscorr(gy,gm,4);

corr_gy_i_model =[AC4(19,22); AC3(19,22); AC2(19,22); AC1(19,22); CC(19,22); AC1(22,19); AC2(22,19); AC3(22,19); AC4(22,19)]; 
[corr_gy_i_data,lags] = crosscorr(gy,i,4);

corr_gy_rb_model =[AC4(19,2); AC3(19,2); AC2(19,2); AC1(19,2); CC(19,2); AC1(2,19); AC2(2,19); AC3(2,19); AC4(2,19)]; 
[corr_gy_rb_data,lags] = crosscorr(gy,rb,4);

corr_gy_re_model =[AC4(19,1); AC3(19,1); AC2(19,1); AC1(19,1); CC(19,1); AC1(1,19); AC2(1,19); AC3(1,19); AC4(1,19)]; 
[corr_gy_re_data,lags] = crosscorr(gy,re,4);

corr_gy_py_model =[AC4(19,16); AC3(19,16); AC2(19,16); AC1(19,16); CC(19,16); AC1(16,19); AC2(16,19); AC3(16,19); AC4(16,19)]; 
[corr_gy_py_data,lags] = crosscorr(gy,py,4);

%Re(1) Rb(2) py(16) w qe gy(19) gm(20) v(21) i(22) r(23); 
corr_gm_gm_model =[AC4(20,20); AC3(20,20); AC2(20,20); AC1(20,20); CC(20,20); AC1(20,20); AC2(20,20); AC3(20,20); AC4(20,20)]; 
[corr_gm_gm_data,lags] = crosscorr(gm,gm,4);

corr_gm_i_model =[AC4(20,22); AC3(20,22); AC2(20,22); AC1(20,22); CC(20,22); AC1(22,20); AC2(22,20); AC3(22,20); AC4(22,20)]; 
[corr_gm_i_data,lags] = crosscorr(gm,i,4);

corr_gm_rb_model =[AC4(20,2); AC3(20,2); AC2(20,2); AC1(20,2); CC(20,2); AC1(2,20); AC2(2,20); AC3(2,20); AC4(2,20)]; 
[corr_gm_rb_data,lags] = crosscorr(gm,rb,4);

corr_gm_re_model =[AC4(20,1); AC3(20,1); AC2(20,1); AC1(20,1); CC(20,1); AC1(1,20); AC2(1,20); AC3(1,20); AC4(1,20)]; 
[corr_gm_re_data,lags] = crosscorr(gm,re,4);

corr_gm_py_model =[AC4(20,16); AC3(20,16); AC2(20,16); AC1(20,16); CC(20,16); AC1(16,20); AC2(16,20); AC3(16,20); AC4(16,20)]; 
[corr_gm_py_data,lags] = crosscorr(gm,py,4);

figure
stem(lags,[corr_gy_gy_data],'r','filled');
hold on
stem(lags,[corr_gy_gy_model],'-b');
title('$corr(\gamma_{yt},\gamma_{yt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gy_gm_data],'r','filled');
hold on
stem(lags,[corr_gy_gm_model],'-b');
title('$corr(\gamma_{yt},\gamma_{mt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);


figure
stem(lags,[corr_gy_i_data],'r','filled');
hold on
stem(lags,[corr_gy_i_model],'-b');
title('$corr(\gamma_{yt},i_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gy_rb_data],'r','filled');
hold on
stem(lags,[corr_gy_rb_model],'-b');
title('$corr(\gamma_{yt},r_{bt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);


figure
stem(lags,[corr_gy_re_data],'r','filled');
hold on
stem(lags,[corr_gy_re_model],'-b');
title('$corr(\gamma_{yt},r_{et\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gy_py_data],'r','filled');
hold on
stem(lags,[corr_gy_py_model],'-b');
title('$corr(\gamma_{yt},\pi_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gm_gm_data],'r','filled');
hold on
stem(lags,[corr_gm_gm_model],'-b');
title('$corr(\gamma_{mt},\gamma_{mt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);


figure
stem(lags,[corr_gm_i_data],'r','filled');
hold on
stem(lags,[corr_gm_i_model],'-b');
title('$corr(\gamma_{mt},i_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gm_rb_data],'r','filled');
hold on
stem(lags,[corr_gm_rb_model],'-b');
title('$corr(\gamma_{mt},r_{bt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);


figure
stem(lags,[corr_gm_re_data],'r','filled');
hold on
stem(lags,[corr_gm_re_model],'-b');
title('$corr(\gamma_{mt},r_{et\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gm_py_data],'r','filled');
hold on
stem(lags,[corr_gm_py_model],'-b');
title('$corr(\gamma_{mt},\pi_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);


filename = 'D:\correlations.xlsx';           %--excel for full sample

ze = zeros(size(corr_gy_gy_model,1),1);
tablecorr = [corr_gy_gy_data corr_gy_gy_model ze corr_gy_gm_data corr_gy_gm_model ze corr_gy_i_data corr_gy_i_model ze corr_gy_rb_data corr_gy_rb_model ze corr_gy_re_data corr_gy_re_model ze corr_gy_py_data corr_gy_py_model ze corr_gm_gm_data corr_gm_gm_model ze corr_gm_i_data corr_gm_i_model ze corr_gm_rb_data corr_gm_rb_model ze corr_gm_re_data corr_gm_re_model ze corr_gm_py_data corr_gm_py_model ze];
%writematrix(tablecorr,filename,'Sheet',2,'Range','A1');

NN = 20;
%writematrix([gy_em(1:NN)],filename,'Sheet',4,'Range','B1');
%writematrix([gm_ey(1:NN)],filename,'Sheet',4,'Range','E1');