%--- This file solves heterogeneous agent CIA with equity, money growth and government bonds 
%--- 5 liquidity shocks are considered. sigma = 2
%pause(1/2)

var Re Rb Rm  
    dz1dm dz2dm dz3dm dz4dm  
    z1 z2 z3 z4 zm zb d Qb py w qe gy gm v i r; 

varexo   ey em ;

parameters bbeta t1 t2 t3 t4 t5 p1 p2 p3 p4 p5 sigma w_bar Qb_bar zb_bar gy_bar gm_bar rho_y rho_m rho_ym rho_my f sigma_y sigma_m sigma_ym;


load params_model;
set_param_value('bbeta',bbeta);
set_param_value('t1',t1);
set_param_value('t2',t2);
set_param_value('t3',t3);
set_param_value('t4',t4);
set_param_value('t5',t5);
set_param_value('p1',p1);
set_param_value('p1',p1);
set_param_value('p2',p2);
set_param_value('p3',p3);
set_param_value('p4',p4);
set_param_value('p5',p5);
set_param_value('sigma',sigma);
set_param_value('gm_bar',gm_bar);
set_param_value('gy_bar',gy_bar);
set_param_value('zb_bar',zb_bar);
set_param_value('w_bar',w_bar);
set_param_value('Qb_bar',Qb_bar);
set_param_value('rho_y',rho_y);
set_param_value('rho_m',rho_m);
set_param_value('rho_ym',rho_ym);
set_param_value('rho_my',rho_my);
set_param_value('sigma_y', sigma_y);
set_param_value('sigma_m', sigma_m);
set_param_value('sigma_ym', sigma_ym);
set_param_value('f', f);

model;


%--- eq 1 definition of Re
Re   = (qe*(1+gy)+1/(1+py))/qe(-1);

%--- eq 2 definition of Rb
Rb   = 1/((1+py)*Qb(-1));

%--- eq 3 definition of rm
Rm   = 1/(1+py);

%--- eq 4 definition of dz1dm
dz1dm = (t1*zm^(-(sigma+1))*(1-z1)^(-sigma)+bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z1*zm)^(-(sigma+1))*Rm(+1)*(Re(+1)-Rm(+1)*z1))
        /(t1*(1-z1)^(-(sigma+1))*zm^(-sigma)+bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z1*zm)^(-(sigma+1))*Rm(+1)^2*zm); 

%--- eq 5 definition of dz2dm
dz2dm = (t2*zm^(-(sigma+1))*(1-z2)^(-sigma)+bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z2*zm)^(-(sigma+1))*Rm(+1)*(Re(+1)-Rm(+1)*z2))
        /(t2*(1-z2)^(-(sigma+1))*zm^(-sigma)+bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z2*zm)^(-(sigma+1))*Rm(+1)^2*zm); 

%--- eq 6 definition of dz3dm
dz3dm = (t3*zm^(-(sigma+1))*(1-z3)^(-sigma)+bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z3*zm)^(-(sigma+1))*Rm(+1)*(Re(+1)-Rm(+1)*z3))
        /(t3*(1-z3)^(-(sigma+1))*zm^(-sigma)+bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z3*zm)^(-(sigma+1))*Rm(+1)^2*zm); 
        
%--- eq 7 definition of dz4dm
dz4dm = (t4*zm^(-(sigma+1))*(1-z4)^(-sigma)+bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z4*zm)^(-(sigma+1))*Rm(+1)*(Re(+1)-Rm(+1)*z4))
        /(t4*(1-z4)^(-(sigma+1))*zm^(-sigma)+bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z4*zm)^(-(sigma+1))*Rm(+1)^2*zm); 
        
        
%--- eq 12 optimality of savings rate z1
t1*((1-z1)*zm)^(-sigma) = bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z1*zm)^(-sigma)*Rm(+1);

%--- eq 13 optimality of savings rate z2
t2*((1-z2)*zm)^(-sigma) = bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z2*zm)^(-sigma)*Rm(+1);

%--- eq 14 optimality of savings rate z3
t3*((1-z3)*zm)^(-sigma) = bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z3*zm)^(-sigma)*Rm(+1);

%--- eq 15 optimality of savings rate z4
t4*((1-z4)*zm)^(-sigma) = bbeta*d(+1)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z4*zm)^(-sigma)*Rm(+1);

%--- eq 16 FOC with respect to zm
p1*t1*((1-z1)*zm)^(-sigma)*(-dz1dm*zm+1-z1)+
p2*t2*((1-z2)*zm)^(-sigma)*(-dz2dm*zm+1-z2)+
p3*t3*((1-z3)*zm)^(-sigma)*(-dz3dm*zm+1-z3)+
p4*t4*((1-z4)*zm)^(-sigma)*(-dz4dm*zm+1-z4)+t5*p5*zm^(-sigma)-
bbeta*d(+1)*(p1*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z1*zm)^(-sigma)*(Re(+1)-Rm(+1)*(z1+dz1dm*zm))+p2*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z2*zm)^(-sigma)*(Re(+1)-Rm(+1)*(z2+dz2dm*zm))+p3*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z3*zm)^(-sigma)*(Re(+1)-Rm(+1)*(z3+dz3dm*zm))+p4*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z4*zm)^(-sigma)*(Re(+1)-Rm(+1)*(z4+dz4dm*zm))+p5*(Re(+1)*(1-zb-zm)+Rb(+1)*zb)^(-sigma)*Re(+1))=0;

%--- eq 17 FOC with respect to zb

Rb(+1)-Re(+1)=0;

%--- eq 18 market clearing goods
1 = ((1-z1)*p1+(1-z2)*p2+(1-z3)*p3+(1-z4)*p4+p5)*zm*w;

%-- eq 19 market clearing equity
qe = (1-zb-zm)*w;

%--- eq 20 government budget constraint
(zb(-1)/Qb(-1)+zm(-1))*w(-1) = (1+py)*(zm+zb)*w*(1+gy);

%--- eq 21 money creation
(1+py)*zm*w*(1+gy) = (1+gm)*zm(-1)*w(-1);

%--- eq 22 d
d = p1*t1*((1-z1)*zm)^(1-sigma)+
    p2*t2*((1-z2)*zm)^(1-sigma)+
    p3*t3*((1-z3)*zm)^(1-sigma)+
    p4*t4*((1-z4)*zm)^(1-sigma)+
    p5*t5*((1-0)*zm)^(1-sigma)+
    bbeta*d(+1)*(p1*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z1*zm)^(1-sigma)+p2*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z2*zm)^(1-sigma)+p3*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z3*zm)^(1-sigma)+p4*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z4*zm)^(1-sigma)+p5*(Re(+1)*(1-zb-zm)+Rb(+1)*zb)^(1-sigma));

%--- Eq 23 dividends
gy = gy_bar +rho_y*(gy(-1)-gy_bar)+rho_ym*(gm(-1)-gm_bar)+ey;

%--- Eq 24 rate of money growth
gm = gm_bar +rho_m*(gm(-1)-gm_bar)+rho_my*(gy(-1)-gy_bar)+ f*(zb(-1)*w(-1)/Qb(-1)-zb_bar*w_bar/Qb_bar)+em;

%-- Eq 25 Velocity
v = 1/(zm*w);

%-- Eq 26 Nominal rates
i = 1/Qb-1;

%-- Eq 27 Real rates
r = (1+i)/(1+py)-1;

end;

initval;
Re = a(18); 
Rb = a(19);
Rm = a(20);
dz1dm = a(21); 
dz2dm = a(22); 
dz3dm = a(23); 
dz4dm = a(24);
z1 = a(25); 
z2 = a(26); 
z3 = a(27); 
z4 = a(28);  
zm = a(29); 
d  = a(30);
py = a(31);
qe = a(32);

zb = zb_bar;
Qb = Qb_bar;
w  = w_bar;
gy = gy_bar;
gm = gm_bar;
v  = 1/(zm*w);
i  = 1/Qb-1;
r  = (1+i)/(1+py)-1;
end;

steady(solve_algo=2);

check;

shocks;
var ey = sigma_y^2;
var em = sigma_m^2;
corr ey,em = sigma_ym;
end;

stoch_simul(irf=30,order=1,contemporaneous_correlation,nograph,pruning);