%--- This file solves heterogeneous agent CIA with equity, money growth and government bonds 
%--- 5 liquidity shocks are considered. sigma = 3
%--- Growth in dividends is considered
pause(1/2)
var Re Rb Rm  
    dz1dm dz2dm dz3dm dz4dm  
    z1 z2 z3 z4 zm zb d Qb py w qe gy gm v i r; 

varexo  em ey;

parameters bbeta t1 t2 t3 t4 t5 p1 p2 p3 p4 p5 sigma w_bar Qb_bar zb_bar gy_bar gm_bar rho_y rho_m rho_ym rho_my f sigma_y sigma_m sigma_ym;

bbeta = 1.0046331;
t1 = 0.56979591; 
t2 = 0.75484827; 
t3 = 1.0000000; 
t4 = 1.3247695; 
t5 = 1.7550143;
p1 = 0.066807201; 
p2 = 0.24173034; 
p3 = 0.38292492; 
p4 = 0.24173034; 
p5 = 0.066807201;
sigma  = 3;
gm_bar  = 0.013860104;
gy_bar  = 0.0039062522;
zb_bar = 0.015222780;
w_bar  = 316.14469;
Qb_bar = 0.98320325;

%1984-2007							
rho_y   = -0.874901493238087;
sigma_y = 0.0403870840150894;
rho_m   = 0.814117686919835;
sigma_m = 0.0416991764878679;
rho_my  = -0.975780929019133;
rho_ym  = 0.30906911297611;
sigma_ym= 0.0073787907575839; 
f       = 0.21;

												

model;

%--- eq 1 definition of Re
Re   = (qe*(1+gy)+1/(1+py))/qe(-1);

%--- eq 2 definition of Rb
Rb   = 1/((1+py)*Qb(-1));

%--- eq 3 definition of rm
Rm   = 1/(1+py);

%--- eq 4 definition of dz1dm
dz1dm = (t1*zm^(-(sigma+1))*(1-z1)^(-sigma)+bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z1*zm)^(-(sigma+1))*Rm(+1)*(Re(+1)-Rm(+1)*z1))
        /(t1*(1-z1)^(-(sigma+1))*zm^(-sigma)+bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z1*zm)^(-(sigma+1))*Rm(+1)^2*zm); 

%--- eq 5 definition of dz2dm
dz2dm = (t2*zm^(-(sigma+1))*(1-z2)^(-sigma)+bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z2*zm)^(-(sigma+1))*Rm(+1)*(Re(+1)-Rm(+1)*z2))
        /(t2*(1-z2)^(-(sigma+1))*zm^(-sigma)+bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z2*zm)^(-(sigma+1))*Rm(+1)^2*zm); 

%--- eq 6 definition of dz3dm
dz3dm = (t3*zm^(-(sigma+1))*(1-z3)^(-sigma)+bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z3*zm)^(-(sigma+1))*Rm(+1)*(Re(+1)-Rm(+1)*z3))
        /(t3*(1-z3)^(-(sigma+1))*zm^(-sigma)+bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z3*zm)^(-(sigma+1))*Rm(+1)^2*zm); 
        
%--- eq 7 definition of dz4dm
dz4dm = (t4*zm^(-(sigma+1))*(1-z4)^(-sigma)+bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z4*zm)^(-(sigma+1))*Rm(+1)*(Re(+1)-Rm(+1)*z4))
        /(t4*(1-z4)^(-(sigma+1))*zm^(-sigma)+bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z4*zm)^(-(sigma+1))*Rm(+1)^2*zm); 
        
        
%--- eq 12 optimality of savings rate z1
t1*((1-z1)*zm)^(-sigma) = bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z1*zm)^(-sigma)*Rm(+1);

%--- eq 13 optimality of savings rate z2
t2*((1-z2)*zm)^(-sigma) = bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z2*zm)^(-sigma)*Rm(+1);

%--- eq 14 optimality of savings rate z3
t3*((1-z3)*zm)^(-sigma) = bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z3*zm)^(-sigma)*Rm(+1);

%--- eq 15 optimality of savings rate z4
t4*((1-z4)*zm)^(-sigma) = bbeta*d(+1)^(1-sigma)*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z4*zm)^(-sigma)*Rm(+1);

%--- eq 16 FOC with respect to zm
p1*t1*((1-z1)*zm)^(-sigma)*(-dz1dm*zm+1-z1)+
p2*t2*((1-z2)*zm)^(-sigma)*(-dz2dm*zm+1-z2)+
p3*t3*((1-z3)*zm)^(-sigma)*(-dz3dm*zm+1-z3)+
p4*t4*((1-z4)*zm)^(-sigma)*(-dz4dm*zm+1-z4)+t5*p5*zm^(-sigma)-
bbeta*d(+1)^(1-sigma)*(p1*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z1*zm)^(-sigma)*(Re(+1)-Rm(+1)*(z1+dz1dm*zm))+p2*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z2*zm)^(-sigma)*(Re(+1)-Rm(+1)*(z2+dz2dm*zm))+p3*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z3*zm)^(-sigma)*(Re(+1)-Rm(+1)*(z3+dz3dm*zm))+p4*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z4*zm)^(-sigma)*(Re(+1)-Rm(+1)*(z4+dz4dm*zm))+p5*(Re(+1)*(1-zb-zm)+Rb(+1)*zb)^(-sigma)*Re(+1))=0;

%--- eq 17 FOC with respect to zb
Rb(+1)-Re(+1)=0;

%--- eq 18 market clearing goods
1 = ((1-z1)*p1+(1-z2)*p2+(1-z3)*p3+(1-z4)*p4+p5)*zm*w;

%-- eq 19 market clearing equity
qe = (1-zb-zm)*w;

%--- eq 20 government budget constraint
(zb(-1)/Qb(-1)+zm(-1))*w(-1) = (1+py)*(zm+zb)*w*(1+gy);

%--- eq 21 money creation
(1+py)*zm*w*(1+gy) = (1+gm)*zm(-1)*w(-1);

%--- eq 22 d
d^(1-sigma) = p1*t1*((1-z1)*zm)^(1-sigma)+
    p2*t2*((1-z2)*zm)^(1-sigma)+
    p3*t3*((1-z3)*zm)^(1-sigma)+
    p4*t4*((1-z4)*zm)^(1-sigma)+
    p5*t5*((1-0)*zm)^(1-sigma)+
    bbeta*d(+1)^(1-sigma)*(p1*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z1*zm)^(1-sigma)+p2*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z2*zm)^(1-sigma)+p3*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z3*zm)^(1-sigma)+p4*(Re(+1)*(1-zb-zm)+Rb(+1)*zb+Rm(+1)*z4*zm)^(1-sigma)+p5*(Re(+1)*(1-zb-zm)+Rb(+1)*zb)^(1-sigma));

%--- Eq 23 dividends
gy = gy_bar +rho_y*(gy(-1)-gy_bar)+rho_ym*(gm(-1)-gm_bar)+ey;

%--- Eq 24 rate of money growth
gm = gm_bar +rho_m*(gm(-1)-gm_bar)+rho_my*(gy(-1)-gy_bar)+ f*(zb(-1)*w(-1)/Qb(-1)-zb_bar*w_bar/Qb_bar)+em;

%-- Eq 25 Velocity
v = 1/(zm*w);

%-- Eq 26 Nominal rates
i = 1/Qb-1;

%-- Eq 27 Real rates
r = (1+i)/(1+py)-1;

end;

initval;
Re = 1.0070982; 
Rb = Re;
Rm = 0.99018222;
dz1dm = 209.61688; 
dz2dm = 230.15978; 
dz3dm = 252.70965; 
dz4dm = 277.46126;

z1 = 0.25789004; 
z2 = 0.18516167; 
z3 = 0.10532802; 
z4 = 0.017699508; 

zm = 0.0035405264; 
zb = zb_bar;
d  = 0.00017677546;
Qb = Qb_bar;
py = 0.0099151204;
w  = w_bar;
qe = 310.21277;
gy = gy_bar;
gm = gm_bar;
v  = 1/(zm*w);
i  = 1/Qb-1;
end;

steady(solve_algo=3);

check;

shocks;
var ey = sigma_y^2;
var em = sigma_m^2;
corr ey,em = sigma_ym;

end;

stoch_simul(irf=20,order=2,contemporaneous_correlation,pruning);