%--- This file solves representative agent CIA with equity, money growth and government bonds 


var Re Rb Rm  
    zm zb d Qb py w qe gy gm i r; 

varexo   em ey ;

parameters bbeta t5 sigma w_bar Qb_bar zb_bar gy_bar gm_bar rho_y rho_m rho_ym rho_my f sigma_y sigma_m sigma_ym;


load params_model;
set_param_value('bbeta',bbeta);
set_param_value('t5',t5);
set_param_value('sigma',sigma);
set_param_value('gm_bar',gm_bar);
set_param_value('gy_bar',gy_bar);
set_param_value('zb_bar',zb_bar);
set_param_value('w_bar',w_bar);
set_param_value('Qb_bar',Qb_bar);
set_param_value('rho_y',rho_y);
set_param_value('rho_m',rho_m);
set_param_value('rho_ym',rho_ym);
set_param_value('rho_my',rho_my);
set_param_value('sigma_y', sigma_y);
set_param_value('sigma_m', sigma_m);
set_param_value('sigma_ym', sigma_ym);
set_param_value('f', f);

model;

%--- eq 1 definition of Re
Re   = (qe*(1+gy)+1/(1+py))/qe(-1);

%--- eq 2 definition of Rb
Rb   = 1/((1+py)*Qb(-1));

%--- eq 3 definition of rm
Rm   = 1/(1+py);

%--- eq 4 FOC with respect to zm
t5*zm^(-sigma)-
bbeta*d(+1)*((Re(+1)*(1-zb-zm)+Rb(+1)*zb)^(-sigma)*Re(+1))=0;

%--- eq 5 FOC with respect to zb
d(+1)*(Rb(+1)-Re(+1))*((Re(+1)*(1-zb-zm)+Rb(+1)*zb)^(-sigma))=0;

%--- eq 6 market clearing goods
1 = zm*w;

%-- eq 7 market clearing equity
qe = (1-zb-zm)*w;

%--- eq 8 government budget constraint
(zb(-1)/Qb(-1)+zm(-1))*w(-1) = (1+py)*(zm+zb)*w*(1+gy);

%--- eq 9 money creation
(1+py)*zm*w*(1+gy) = (1+gm)*zm(-1)*w(-1);

%--- eq 10 d
d = t5*((1-0)*zm)^(1-sigma)+
    bbeta*d(+1)*((Re(+1)*(1-zb-zm)+Rb(+1)*zb)^(1-sigma));

%--- Eq 11 dividends
gy = gy_bar +rho_y*(gy(-1)-gy_bar)+rho_ym*(gm(-1)-gm_bar)+ey;

%--- Eq 12 rate of money growth
gm = gm_bar +rho_m*(gm(-1)-gm_bar)+rho_my*(gy(-1)-gy_bar)+ f*(zb(-1)*w(-1)/Qb(-1)-zb_bar*w_bar/Qb_bar)+em;

%-- Eq 13 Nominal rates
i = 1/Qb-1;

%-- Eq 14 Real rates
r = (1+i)/(1+py)-1;


end;

initval;
Re = 1.01745; 
Rb = Re;
Rm = 0.99018222;

zm = 0.013311463; 
zb = zb_bar;
d  = 149.39723;
Qb = Qb_bar;
py = 0.0099151204;
w  = w_bar;
qe = 73.109913 ;
gy = gy_bar;
gm = gm_bar;
i  = 1/Qb-1;
end;

steady(solve_algo=2);

check;

shocks;
var ey = sigma_y^2;
var em = sigma_m^2;
corr ey,em = sigma_ym;
end;

stoch_simul(irf=30,order=1,contemporaneous_correlation,nograph,pruning);