%% Calibration code %%

% date: 04-08-2020
% author: lucas rosso

clear all; close all; clc;

path = 'D:\Dropbox\Brennan\new projects\money demand new\revision\data\main data.xls';
data = xlsread(path, 12,'B3:I254');
datas = data;            %--data for FULL SAMPLE

NN = size(datas,1);


%%%% MOMENTS FROM THE DATA USED FOR ESTIMATION(SEE MAIN DATA.XLS) %%%
gy_mean = mean(datas(:,1));
gm_mean = mean(datas(:,2));
sd_gy = std(datas(:,1));
sd_gm = std(datas(:,2));

corr_gy_gyl = corr(datas(2:NN,1),datas(1:NN-1,1));
corr_gm_gml = corr(datas(2:NN,2),datas(1:NN-1,2));
corr_gy_gml = corr(datas(2:NN,1),datas(1:NN-1,2));
corr_gm_gyl = corr(datas(2:NN,2),datas(1:NN-1,1));
corr_gy_gm  = corr(datas(:,1),datas(:,2));

%%%% MOMENTS FROM THE DATA USED FOR COMPUTATION %%%
gy       = datas(:,1);
gm       = datas(:,2);
velocity = datas(:,8);
rb       = datas(:,5);
py       = datas(:,3);
re       = datas(:,6);
qe       = datas(:,7);
i        = datas(:,4);

ret_mean = 0.01745;

py_bar = (1+gm_mean)/(1+gy_mean)-1;
Rm_bar = 1/(1+py_bar);
qe_bar = 1/((ret_mean-gy_mean)*(1+py_bar));
Qb_bar = 1/((1+py_bar)*(1+ret_mean));
i_bar  = 1/Qb_bar-1;

%removing trends

py = detrend(py,0)+py_bar;

i = detrend(i,0)+i_bar;

rb = detrend(rb,0)+ret_mean;

velocity= velocity-hpfilter(velocity,1600);



sd_i        = std(i);
sd_py       = std(py);
sd_velocity = std(velocity);
sd_rb       = std(rb);
sd_py       = std(py);


%---constructing re as in the model
qe = qe-hpfilter(qe,1600);
qe_hat = qe+qe_bar;

re = (qe_hat(2:size(qe_hat)).*(1+datas(2:size(qe_hat),1))+1./(1+datas(2:size(qe_hat),3)))./qe_hat(1:size(qe_hat)-1)-1;
sd_re = std(re);


%--Here I compute relevant statistics in tables
filename = 'D:\Tablasfull.xlsx';           %--excel for full sample



global data_moments oo_
% parameter guess

rho_y   = -0.078581594094478;
sigma_y = 0.009631189638213;
rho_m   = 0.384245152150355;
sigma_m = 0.009377923455828;
rho_my  = 0.079016233265885;
rho_ym  = 0.151037420353135;
sigma_ym= -0.590189376906576; 
f = 0.012398338658692;

vector_params = [rho_y,sigma_y,rho_m,sigma_m,rho_my,rho_ym,sigma_ym,f];

data_moments = [sd_gy,corr_gy_gyl,sd_gm,corr_gm_gml,corr_gm_gyl,corr_gy_gml,corr_gy_gm]; 


%    [rho_y,sigma_y,rho_g,sigma_g,rho_gy,rho_yg,sigma_yg ,   f  ];

LB = [-0.95  ,1e-10  ,-0.9  ,1e-10  ,-2    ,-0.2  ,-0.99     ,0.01   ];
UB = [0.95  ,0.01   ,0.9  ,0.01   ,5     ,0.2   ,0.99      ,0.2    ];

%LB = [0.2  ,1e-10  ,-0.9  ,1e-10  ,-2    ,-0.2  ,-0.99     ,0.01   ];
%UB = [0.95  ,0.01   ,0.9  ,0.01   ,7     ,0.2   ,0.99      ,0.5    ];

options = optimset('PlotFcns',@optimplotfval,'Display','iter','MaxFunEvals',10000,'MaxIter',5000);

optimal_params = fminsearchbnd(@Loss, vector_params, LB, UB, options)

%Re(1) py(8) gy(11) gm(12) i(13);

Table = [sqrt(oo_.var(11,11)), data_moments(1);
        oo_.autocorr{1}(11,11), data_moments(2);
        sqrt(oo_.var(12,12)), data_moments(3);
        oo_.autocorr{1}(12,12), data_moments(4);
        oo_.autocorr{1}(12,11), data_moments(5);
        oo_.autocorr{1}(11,12), data_moments(6);
        oo_.contemporaneous_correlation(11,12), data_moments(7)];

save Results optimal_params Table

%Re(1) py(8) gy(11) gm(12) i(13);

MV = oo_.var;
CC = oo_.contemporaneous_correlation;
AC = oo_.autocorr{1,1};

%---"estimated" statistics---%
cgygyl = AC(11,11);
cgygml = AC(11,12);
sdgy   = sqrt(MV(11,11));
cgmgml = AC(12,12);
cgmgyl = AC(12,11);
sdgm   = sqrt(MV(12,12));
cgygm  = CC(11,12);

table2 = [optimal_params]';
%writematrix(table2,filename,'Sheet',2,'Range','F2');

table3 = [cgygyl;cgygml;sdgy;cgmgml;cgmgyl;sdgm;cgygm];
%writematrix(table3,filename,'Sheet',3,'Range','G2');

%---standard deviations---%
%Re(1) py(8) gy(11) gm(12) i(13) r(14);

sdi  = sqrt(MV(13,13));
sdr  = sqrt(MV(14,14));
sdRe = sqrt(MV(1,1));
sdpy = sqrt(MV(8,8));

table4 = [0;sdi;sdr;sdRe;sdpy]*100;
%writematrix(table4,filename,'Sheet',4,'Range','G2');

AC1 = oo_.autocorr{1,1};
AC2 = oo_.autocorr{1,2};
AC3 = oo_.autocorr{1,3};
AC4 = oo_.autocorr{1,4};


%Re(1) py(8) gy(11) gm(12) i(13) r(14);


corr_gy_gy_model =[AC4(11,11); AC3(11,11); AC2(11,11); AC1(11,11); CC(11,11); AC1(11,11); AC2(11,11); AC3(11,11); AC4(11,11)]; 
[corr_gy_gy_data,lags] = crosscorr(gy,gy,4);

corr_gy_gm_model =[AC4(11,12); AC3(11,12); AC2(11,12); AC1(11,12); CC(11,12); AC1(12,11); AC2(12,11); AC3(12,11); AC4(12,11)]; 
[corr_gy_gm_data,lags] = crosscorr(gy,gm,4);

corr_gy_i_model =[AC4(11,13); AC3(11,13); AC2(11,13); AC1(11,13); CC(11,13); AC1(11,13); AC2(13,11); AC3(13,11); AC4(13,11)]; 
[corr_gy_i_data,lags] = crosscorr(gy,i,4);

corr_gy_rb_model =[AC4(11,14); AC3(11,14); AC2(11,14); AC1(11,14); CC(11,14); AC1(14,11); AC2(14,11); AC3(14,11); AC4(14,11)]; 
[corr_gy_rb_data,lags] = crosscorr(gy,rb,4);

corr_gy_re_model =[AC4(11,1); AC3(11,1); AC2(11,1); AC1(11,1); CC(11,1); AC1(1,11); AC2(1,11); AC3(1,11); AC4(1,11)]; 
[corr_gy_re_data,lags] = crosscorr(gy,re,4);

corr_gy_py_model =[AC4(11,8); AC3(11,8); AC2(11,8); AC1(11,8); CC(11,8); AC1(8,11); AC2(8,11); AC3(8,11); AC4(8,11)]; 
[corr_gy_py_data,lags] = crosscorr(gy,py,4);


%Re(1) py(8) gy(11) gm(12) i(13) r(14);

corr_gm_gm_model =[AC4(12,12); AC3(12,12); AC2(12,12); AC1(12,12); CC(12,12); AC1(12,12); AC2(12,12); AC3(12,12); AC4(12,12)]; 
[corr_gm_gm_data,lags] = crosscorr(gm,gm,4);

corr_gm_i_model =[AC4(12,13); AC3(12,13); AC2(12,13); AC1(12,13); CC(12,13); AC1(13,12); AC2(13,12); AC3(13,12); AC4(13,12)]; 
[corr_gm_i_data,lags] = crosscorr(gm,i,4);

corr_gm_rb_model =[AC4(12,14); AC3(12,14); AC2(12,14); AC1(12,14); CC(12,14); AC1(14,12); AC2(14,12); AC3(14,12); AC4(14,12)]; 
[corr_gm_rb_data,lags] = crosscorr(gm,rb,4);

corr_gm_re_model =[AC4(12,1); AC3(12,1); AC2(12,1); AC1(12,1); CC(12,1); AC1(1,12); AC2(1,12); AC3(1,12); AC4(1,12)]; 
[corr_gm_re_data,lags] = crosscorr(gm,re,4);

corr_gm_py_model =[AC4(12,8); AC3(12,8); AC2(12,8); AC1(12,8); CC(12,8); AC1(8,12); AC2(8,12); AC3(8,12); AC4(8,12)]; 
[corr_gm_py_data,lags] = crosscorr(gm,py,4);

figure
stem(lags,[corr_gy_gy_data],'r','filled');
hold on
stem(lags,[corr_gy_gy_model],'-b');
title('$corr(\gamma_{yt},\gamma_{yt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gy_gm_data],'r','filled');
hold on
stem(lags,[corr_gy_gm_model],'-b');
title('$corr(\gamma_{yt},\gamma_{mt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);


figure
stem(lags,[corr_gy_i_data],'r','filled');
hold on
stem(lags,[corr_gy_i_model],'-b');
title('$corr(\gamma_{yt},i_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gy_rb_data],'r','filled');
hold on
stem(lags,[corr_gy_rb_model],'-b');
title('$corr(\gamma_{yt},r_{bt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);


figure
stem(lags,[corr_gy_re_data],'r','filled');
hold on
stem(lags,[corr_gy_re_model],'-b');
title('$corr(\gamma_{yt},r_{et\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gy_py_data],'r','filled');
hold on
stem(lags,[corr_gy_py_model],'-b');
title('$corr(\gamma_{yt},\pi_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gm_gm_data],'r','filled');
hold on
stem(lags,[corr_gm_gm_model],'-b');
title('$corr(\gamma_{mt},\gamma_{mt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);


figure
stem(lags,[corr_gm_i_data],'r','filled');
hold on
stem(lags,[corr_gm_i_model],'-b');
title('$corr(\gamma_{mt},i_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gm_rb_data],'r','filled');
hold on
stem(lags,[corr_gm_rb_model],'-b');
title('$corr(\gamma_{mt},r_{bt\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);


figure
stem(lags,[corr_gm_re_data],'r','filled');
hold on
stem(lags,[corr_gm_re_model],'-b');
title('$corr(\gamma_{mt},r_{et\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);

figure
stem(lags,[corr_gm_py_data],'r','filled');
hold on
stem(lags,[corr_gm_py_model],'-b');
title('$corr(\gamma_{mt},\pi_{t\pm\ell}$)','Interpreter','latex','FontSize',20);
legend('Data', 'Model', 'Location', 'SouthEast','Fontsize',14);
ylim([-1 1]);


filename = 'D:\correlations.xlsx';           %--excel for full sample


writematrix(corr_gy_gy_model,filename,'Sheet',2,'Range','C1');
writematrix(corr_gy_gm_model,filename,'Sheet',2,'Range','F1');
writematrix(corr_gy_i_model,filename,'Sheet',2,'Range','I1');
writematrix(corr_gy_rb_model,filename,'Sheet',2,'Range','L1');
writematrix(corr_gy_re_model,filename,'Sheet',2,'Range','O1');
writematrix(corr_gy_py_model,filename,'Sheet',2,'Range','R1');
writematrix(corr_gm_gm_model,filename,'Sheet',2,'Range','U1');
writematrix(corr_gm_i_model,filename,'Sheet',2,'Range','X1');
writematrix(corr_gm_rb_model,filename,'Sheet',2,'Range','AA1');
writematrix(corr_gm_re_model,filename,'Sheet',2,'Range','AD1');
writematrix(corr_gm_py_model,filename,'Sheet',2,'Range','AG1')  ;

