%--- This file solves heterogeneous agent CIA with equity, money growth and government bonds 
%--- 5 liquidity shocks are considered. sigma = 3
%--- Growth in dividends is considered
pause(1/2)

var Re Rb Rm  
    zm zb d Qb py w qe gy gm i r; 

varexo  em ey;

parameters bbeta t5 sigma w_bar Qb_bar zb_bar gy_bar gm_bar rho_y rho_m rho_ym rho_my f sigma_y sigma_m sigma_ym;

bbeta = 1.0019540;
t5 = 2.8654070;
sigma  = 3;
gm_bar  = 0.012836922;
gy_bar  = 0.0046178362;
zb_bar = 0.012582514;
w_bar  = 138.19859;
Qb_bar = 0.98018152;


%1984-2007
rho_y   = -0.027499382408105;
sigma_y = 0.003593929914167;
rho_m   = 0.224302120398765;
sigma_m = 0.003568146058977;
rho_my  = -0.270301020326941;
rho_ym  = -0.078242993991275;
sigma_ym= 0.079346824161888; 
f       = 0.103255489346494;

model;

%--- eq 1 definition of Re
Re   = (qe*(1+gy)+1/(1+py))/qe(-1);

%--- eq 2 definition of Rb
Rb   = 1/((1+py)*Qb(-1));

%--- eq 3 definition of rm
Rm   = 1/(1+py);

%--- eq 4 FOC with respect to zm
t5*zm^(-sigma)-
bbeta*d(+1)^(1-sigma)*((Re(+1)*(1-zb-zm)+Rb(+1)*zb)^(-sigma)*Re(+1))=0;

%--- eq 5 FOC with respect to zb
d(+1)^(1-sigma)*(Rb(+1)-Re(+1))*((Re(+1)*(1-zb-zm)+Rb(+1)*zb)^(-sigma))=0;

%--- eq 6 market clearing goods
1 = zm*w;

%-- eq 7 market clearing equity
qe = (1-zb-zm)*w;

%--- eq 8 government budget constraint
(zb(-1)/Qb(-1)+zm(-1))*w(-1) = (1+py)*(zm+zb)*w*(1+gy);

%--- eq 9 money creation
(1+py)*zm*w*(1+gy) = (1+gm)*zm(-1)*w(-1);

%--- eq 10 d
d = (t5*((1-0)*zm)^(1-sigma)+
    bbeta*d(+1)^(1-sigma)*((Re(+1)*(1-zb-zm)+Rb(+1)*zb)^(1-sigma)))^(1/(1-sigma));

%--- Eq 11 dividends
gy = gy_bar +rho_y*(gy(-1)-gy_bar)+rho_ym*(gm(-1)-gm_bar)+ey;

%--- Eq 12 rate of money growth
gm = gm_bar +rho_m*(gm(-1)-gm_bar)+rho_my*(gy(-1)-gy_bar)+ f*(zb(-1)*w(-1)/Qb(-1)-zb_bar*w_bar/Qb_bar)+em;

%-- Eq 13 Nominal rates
i = 1/Qb-1;

%-- Eq 14 Real rates
r = (1+i)/(1+py)-1;

end;

initval;
Re = 1.0119402; 
Rb = Re;
Rm = 0.99188508;

zm = 0.0072359640; 
zb = zb_bar;
d  = 0.00036362307;
Qb = Qb_bar;
py = 0.0081813059;
w  = w_bar;
qe = 135.45970;
gy = gy_bar;
gm = gm_bar;
i  = 1/Qb-1;
end;

steady(solve_algo=3);

check;

shocks;
var ey = sigma_y^2;
var em = sigma_m^2;
corr ey,em = sigma_ym;

end;

stoch_simul(irf=20,order=1,contemporaneous_correlation,pruning);